import java.io.IOException;
import java.io.InputStream;
import java.nio.FloatBuffer;

import javax.swing.*;
import javax.media.opengl.*;
import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;

import com.jogamp.opengl.util.texture.Texture;
import com.jogamp.opengl.util.texture.TextureData;
import com.jogamp.opengl.util.texture.TextureIO;



public class JOGL_Light_and_Material extends JFrame{

	GLCanvas canvas;
	SceneView sv;

	public JOGL_Light_and_Material() {

		sv = new SceneView(); //GLEventListener-Objekt

		GLProfile glp = GLProfile.getDefault();
		GLCapabilities caps = new GLCapabilities(glp);
		canvas = new GLCanvas(caps);

		canvas.addGLEventListener(sv);

		add(canvas);

		setTitle("Jogl Light and Material");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(500,500);
		setVisible(true);  

	}



	class SceneView implements GLEventListener {

		private Texture texture1;
		private Texture texture2;

		public void init(GLAutoDrawable drawable) {
			GL2 gl = drawable.getGL().getGL2(); 
			GLU glu = new GLU();
			gl.glClearColor(0.5f, 0.5f, 0.5f, 0f); // grey background	
			gl.glMatrixMode(GL2.GL_PROJECTION);
			gl.glLoadIdentity();
			glu.gluPerspective(45, 1, 0.1, 100);

			//light setup
			float light_ambient[] = { 0.2f, 0.2f, 0.2f, 1.0f };
			float light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float light_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float light_position[] = { -10f, -6f, 10.0f, 0.0f };
			float light_direction[] = { 10f, 5f, -10.0f, 0.0f };


			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_AMBIENT, FloatBuffer.wrap(light_ambient));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_DIFFUSE, FloatBuffer.wrap(light_diffuse));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_SPECULAR, FloatBuffer.wrap(light_specular));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_POSITION, FloatBuffer.wrap(light_position));
			gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_SPOT_DIRECTION, FloatBuffer.wrap(light_direction));
			gl.glLightf(GL2.GL_LIGHT0, GL2.GL_SPOT_CUTOFF, 45.0f);
			gl.glLightf(GL2.GL_LIGHT0, GL2.GL_QUADRATIC_ATTENUATION, 1f);//quadratic falloff

			gl.glEnable(GL2.GL_LIGHTING);
			gl.glEnable(GL2.GL_LIGHT0);

			gl.glEnable(GL.GL_DEPTH_TEST);
			gl.glClearDepth(1.0f);                      
			gl.glDepthFunc(GL.GL_LEQUAL);
			gl.glHint(GL2.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
			gl.glShadeModel(GL2.GL_SMOOTH); //Gouraud-Shading

			//loading textures
			texture1 = loadTexture(texture1,"textur","png",gl); 
			texture2 = loadTexture(texture2,"grass","png",gl); 

			//setup camera
			gl.glMatrixMode(GL2.GL_MODELVIEW);
			gl.glLoadIdentity();
			glu.gluLookAt(0, 0, 3, 0, 0, -5, 0, 1, 0);

		}

		public void display(GLAutoDrawable drawable) {

			GL2 gl = drawable.getGL().getGL2();
			GLU glu = new GLU();

			//quadric with normals and texture coordinates
			GLUquadric q = glu.gluNewQuadric();
			glu.gluQuadricDrawStyle( q, GLU.GLU_FILL );
			glu.gluQuadricNormals( q, GLU.GLU_SMOOTH );
			glu.gluQuadricTexture(q, true);


			gl.glClear(GL2.GL_COLOR_BUFFER_BIT|GL2.GL_DEPTH_BUFFER_BIT); // clear background

			//materials
			float no_mat[] = { 0.0f, 0.1f, 0.1f, 1.0f };
			float mat_ambient[] = { 1f, 1f, 1f, 1.0f };
			float mat_diffuse[] = { 0.1f, 0.5f, 0.8f, 1.0f };
			float mat_diffuse2[] = { 0.8f, 0.8f, 0.8f, 1.0f };
			float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
			float no_shininess[] = { 0.0f };
			float low_shininess[] = { 80.0f };
			

			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_AMBIENT, FloatBuffer.wrap(no_mat));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_DIFFUSE, FloatBuffer.wrap(mat_diffuse));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SPECULAR, FloatBuffer.wrap(mat_specular));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SHININESS, FloatBuffer.wrap(low_shininess));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_EMISSION, FloatBuffer.wrap(no_mat));

			gl.glPushMatrix();
			gl.glTranslated(0, -1, -3);
			glu.gluSphere(q, 0.8f, 200, 200);
			gl.glPopMatrix();

			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_AMBIENT, FloatBuffer.wrap(mat_ambient));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_DIFFUSE, FloatBuffer.wrap(mat_diffuse2));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SPECULAR, FloatBuffer.wrap(no_mat));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SHININESS, FloatBuffer.wrap(no_shininess));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_EMISSION, FloatBuffer.wrap(no_mat));

			texture2.enable(gl);
			texture2.bind(gl);

			gl.glPushMatrix();
			gl.glTranslated(-1.7,-0.2,-6);
			glu.gluSphere(q, 0.8f, 200, 200);

			gl.glPopMatrix();

			texture2.disable(gl);

			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_AMBIENT, FloatBuffer.wrap(no_mat));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_DIFFUSE, FloatBuffer.wrap(mat_diffuse2));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SPECULAR, FloatBuffer.wrap(no_mat));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_SHININESS, FloatBuffer.wrap(no_shininess));
			gl.glMaterialfv(GL2.GL_FRONT, GL2.GL_EMISSION, FloatBuffer.wrap(no_mat));

			texture1.enable(gl);
			texture1.bind(gl);
			gl.glPushMatrix();

			gl.glTranslated(1.5,0,-1);
			gl.glRotated(29, 1, 1, 1);
			gl.glTranslated(-0.5,-0.5,0.5);

			gl.glBegin(GL2.GL_QUADS);

			//Front
			gl.glNormal3d(0, 0, 1);
			gl.glTexCoord2d(0, 0);
			gl.glVertex3d(0,0,0);
			gl.glTexCoord2d(1, 0);
			gl.glVertex3d(0,1,0);
			gl.glTexCoord2d(1, 1);
			gl.glVertex3d(1,1,0);
			gl.glTexCoord2d(0, 1);
			gl.glVertex3d(1,0,0);
			//Back
			gl.glNormal3d(0, 0, -1);
			gl.glTexCoord2d(0, 0);
			gl.glVertex3d(0,0,-1);
			gl.glTexCoord2d(1, 0);
			gl.glVertex3d(0,1,-1);
			gl.glTexCoord2d(1, 1);
			gl.glVertex3d(1,1,-1);
			gl.glTexCoord2d(0, 1);
			gl.glVertex3d(1,0,-1);
			//Bottom
			gl.glNormal3d(0, -1, 0);
			gl.glTexCoord2d(0, 0);
			gl.glVertex3d(0,0,0);
			gl.glTexCoord2d(1, 0);
			gl.glVertex3d(0,0,-1);
			gl.glTexCoord2d(1, 1);
			gl.glVertex3d(1,0,-1);
			gl.glTexCoord2d(0, 1);
			gl.glVertex3d(1,0,0);
			//Top
			gl.glNormal3d(0, 1, 0);
			gl.glTexCoord2d(0, 0);
			gl.glVertex3d(0,1,0);
			gl.glTexCoord2d(1, 0);
			gl.glVertex3d(0,1,-1);
			gl.glTexCoord2d(1, 1);
			gl.glVertex3d(1,1,-1);
			gl.glTexCoord2d(0, 1);
			gl.glVertex3d(1,1,0);
			//Left
			gl.glNormal3d(-1, 0, 0);
			gl.glTexCoord2d(0, 0);
			gl.glVertex3d(0,0,0);
			gl.glTexCoord2d(1, 0);
			gl.glVertex3d(0,1,0);
			gl.glTexCoord2d(1, 1);
			gl.glVertex3d(0,1,-1);
			gl.glTexCoord2d(0, 1);
			gl.glVertex3d(0,0,-1);
			//Right
			gl.glNormal3d(1, 0, 0);
			gl.glTexCoord2d(0, 0);
			gl.glVertex3d(1,0,0);
			gl.glTexCoord2d(1, 0);
			gl.glVertex3d(1,1,0);
			gl.glTexCoord2d(1, 1);
			gl.glVertex3d(1,1,-1);
			gl.glTexCoord2d(0, 1);
			gl.glVertex3d(1,0,-1);

			gl.glEnd();
			gl.glPopMatrix();

			texture1.disable(gl);
		}

		public void reshape(GLAutoDrawable drawable, int x, int y, int w, int h) {
			GL2 gl = drawable.getGL().getGL2();
			GLU glu = new GLU();

			gl.glMatrixMode(GL2.GL_PROJECTION);
			gl.glLoadIdentity();
			glu.gluPerspective(45, w/h, 0.1, 1000);
		}

		public void dispose(GLAutoDrawable drawable) {
		}

		private Texture loadTexture(Texture t, String file, String fileType, GL2 gl){
			try{
				InputStream stream = getClass().getResourceAsStream(file+"."+fileType);
				TextureData data =
						TextureIO.newTextureData(gl.getGLProfile(),stream, false,fileType);
				t = TextureIO.newTexture(data);
				return t;
			}catch(IOException e){
				e.printStackTrace();
				return null;
			}
		}


	}

	public static void main(String args[]) {
		new JOGL_Light_and_Material();   
	}




}
